package com.cj.jiaqingjiayi.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.domain.Business;
import com.cj.jiaqingjiayi.model.domain.Commodities;
import com.cj.jiaqingjiayi.model.domain.User;
import com.cj.jiaqingjiayi.model.request.Commodities.CommoditiesAddRequest;
import com.cj.jiaqingjiayi.model.request.Commodities.CommoditiesDeleteRequest;
import com.cj.jiaqingjiayi.model.request.Commodities.CommoditiesQueryRequest;
import com.cj.jiaqingjiayi.model.request.Commodities.CommoditiesUpdateRequest;
import com.cj.jiaqingjiayi.model.vo.BusinessVO;
import com.cj.jiaqingjiayi.model.vo.CommoditiesVO;
import com.cj.jiaqingjiayi.model.vo.LoginUserVO;
import com.cj.jiaqingjiayi.service.BusinessService;
import com.cj.jiaqingjiayi.service.CommoditiesService;
import com.cj.jiaqingjiayi.service.SpecificationsCommoditiesService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Slf4j
@Api(tags = "商品接口")
@RestController
@RequestMapping("/commodities")
public class CommoditiesController {

    @Resource
    private CommoditiesService commoditiesService;

    @Resource
    private BusinessService businessService;

    @Resource
    private SpecificationsCommoditiesService specificationsCommoditiesService;


    /**
     *添加商品
     * @param commoditiesAddRequest 添加商品请求体
     * @return 是否成功
     */
    @ApiOperation(value = "添加商品")
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> addCommodities (@RequestBody CommoditiesAddRequest commoditiesAddRequest, HttpServletRequest request) {
        if (commoditiesAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Commodities commodities = new Commodities();
        BeanUtils.copyProperties(commoditiesAddRequest,commodities);
        //校验
        commoditiesService.validCommodities(commodities,false);
        //添加商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        commodities.setBusinessId(businessId);
        //保存商品信息
        Long commoditiesId = commoditiesService.addCommodities(commodities);
        //中间表中添加数据
        List<Long> specificationsIds = commoditiesAddRequest.getSpecificationsIds();
        specificationsCommoditiesService.add(commoditiesId,specificationsIds);
        return ResultUtils.success(true);
    }

    /**
     * 删除商品
     * @param commoditiesDeleteRequest 删除请求
     * @param request 网络请求
     * @return 是否成功
     */
    @ApiOperation(value = "删除商品")
    @PostMapping("/delete")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> deleteCommodities(@RequestBody CommoditiesDeleteRequest commoditiesDeleteRequest, HttpServletRequest request) {
        if (commoditiesDeleteRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long commoditiesId = commoditiesDeleteRequest.getId();
        //获取商家id
        Business business = businessService.getLoginBusiness(request);
        Long businessId = business.getId();
        //校验商品是不是当前登录商家的
        LambdaQueryWrapper<Commodities> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Commodities::getId, commoditiesId);
        wrapper.eq(Commodities::getBusinessId, businessId);
        Commodities commodities = commoditiesService.getOne(wrapper);
        ThrowUtils.throwIf(commodities == null, ErrorCode.OPERATION_ERROR, "当前商家无权限删除该商品");
        //删除商品
        boolean commoditiesRemove = commoditiesService.removeById(commoditiesId);
        ThrowUtils.throwIf(!commoditiesRemove, ErrorCode.OPERATION_ERROR);
        //删除中间表信息
        specificationsCommoditiesService.deleteByCommoditiesId(commoditiesId);
        return ResultUtils.success(true);
    }

    /**
     * 更新商品
     * @param commoditiesUpdateRequest 更新请求
     * @param request 网络请求
     * @return 是否成功
     */
    @ApiOperation(value = "更新商品")
    @PostMapping("/update")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> updateCommodities (@RequestBody CommoditiesUpdateRequest commoditiesUpdateRequest, HttpServletRequest request) {
        if (commoditiesUpdateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Commodities commodities = new Commodities();
        BeanUtils.copyProperties(commoditiesUpdateRequest, commodities);
        //校验
        commoditiesService.validCommodities(commodities, true);
        //添加商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        commodities.setBusinessId(businessId);
        //更新商品
        Boolean updateDishes = commoditiesService.updateCommodities(commodities);
        ThrowUtils.throwIf(!updateDishes, ErrorCode.OPERATION_ERROR);
        //更新中间表的信息
        Long commoditiesId = commodities.getId();
        List<Long> specificationsIds = commoditiesUpdateRequest.getSpecificationsIds();
        return ResultUtils.success(specificationsCommoditiesService.updateByCommoditiesId(commoditiesId, specificationsIds));
    }

    /**
     * 更新商品状态
     * @param commoditiesUpdateRequest 商品请求体
     * @param request 前端请求
     * @return 是否成功
     */
    @ApiOperation(value = "更新商品状态")
    @PostMapping("/update/status")
    public BaseResponse<Boolean> updateCommoditiesStatus (@RequestBody CommoditiesUpdateRequest commoditiesUpdateRequest, HttpServletRequest request) {
        Long id = commoditiesUpdateRequest.getId();
        String status = commoditiesUpdateRequest.getStatus();
        if (id == null || StringUtils.isBlank(status)) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR);
        }

        //获取商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        //更新状态
        UpdateWrapper<Commodities> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id",id);
        updateWrapper.eq("businessId",businessId);
        updateWrapper.set("status",status);
        return ResultUtils.success(commoditiesService.update(updateWrapper));
    }

    /**
     *获取商品列表
     * @param commoditiesQueryRequest 查询请求体
     * @return 分页列表
     */
    @ApiOperation(value = "获取商品列表")
    @PostMapping("/list/page/commodities")
    public BaseResponse<Page<Commodities>> commoditiesPage(@RequestBody CommoditiesQueryRequest commoditiesQueryRequest) {
        long current = commoditiesQueryRequest.getCurrent();
        long pageSize = commoditiesQueryRequest.getPageSize();
        //获取查询条件
        QueryWrapper<Commodities> wrapper = commoditiesService.getWrapper(commoditiesQueryRequest);
        //获取商家有的商品
        Page<Commodities> commoditiesPage = commoditiesService.page(new Page<>(current, pageSize), wrapper);
        return ResultUtils.success(commoditiesPage);
    }

//    /**
//     *获取商品脱敏列表
//     * @param commoditiesQueryRequest 查询请求体
//     * @return 分页列表
//     */
//    @ApiOperation(value = "获取商品脱敏列表")
//    @PostMapping("/list/page/commoditiesVO")
//    public BaseResponse<Page<CommoditiesVO>> commoditiesVOPage(@RequestBody CommoditiesQueryRequest commoditiesQueryRequest) {
//        long current = commoditiesQueryRequest.getCurrent();
//        long pageSize = commoditiesQueryRequest.getPageSize();
//        //获取查询条件
//        QueryWrapper<Commodities> wrapper = commoditiesService.getWrapper(commoditiesQueryRequest);
//        //获取商家有的商品
//        Page<Commodities> commoditiesPage = commoditiesService.page(new Page<>(current, pageSize), wrapper);
//        //获取脱敏信息列表
//        List<Commodities> commoditiesList = commoditiesPage.getRecords();
//        long total = commoditiesPage.getTotal();
//        List<CommoditiesVO> commoditiesVOList = commoditiesService.getCommoditiesVOList(commoditiesList);
//        Page<CommoditiesVO> commoditiesVOPage = new Page<>(current, pageSize, total);
//        commoditiesVOPage.setRecords(commoditiesVOList);
//        return ResultUtils.success(commoditiesVOPage);
//    }

    /**
     *根据id获取商品
     */
    @ApiOperation(value = "根据商品id获取商品信息")
    @GetMapping("/getById/commodities")
    public BaseResponse<CommoditiesVO> getCommoditiesById(@RequestParam Long id) {
        ThrowUtils.throwIf(id < 0, ErrorCode.NULL_ERROR);
        Commodities byId = commoditiesService.getById(id);
        CommoditiesVO commoditiesVO = commoditiesService.getCommoditiesVO(byId);
        return ResultUtils.success(commoditiesVO);
    }



}
