package com.cj.jiaqingjiayi.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.domain.Business;
import com.cj.jiaqingjiayi.model.domain.Specifications;

import com.cj.jiaqingjiayi.model.domain.SpecificationsCommodities;
import com.cj.jiaqingjiayi.model.request.attribute.AttributeAddRequest;
import com.cj.jiaqingjiayi.model.request.attribute.AttributeUpdateRequest;
import com.cj.jiaqingjiayi.model.request.specifications.SpecificationsAddRequest;
import com.cj.jiaqingjiayi.model.request.specifications.SpecificationsMyQueryRequest;
import com.cj.jiaqingjiayi.model.request.specifications.SpecificationsQueryRequest;
import com.cj.jiaqingjiayi.model.request.specifications.SpecificationsUpdateRequest;
import com.cj.jiaqingjiayi.model.vo.SpecificationsVO;
import com.cj.jiaqingjiayi.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Slf4j
@Api(tags = "规格接口")
@RestController
@RequestMapping("/specifications")
public class SpecificationsController {

    @Resource
    private SpecificationsService specificationsService;

    @Resource
    private SpecificationsCommoditiesService specificationsCommoditiesService;

    @Resource
    private BusinessService businessService;

    @Resource
    private AttributeService attributeService;

    @Resource
    private UserService userService;

    /**
     * 添加规格
     * @param specificationsAddRequest 添加请求体
     * @param request 网络请求
     * @return 是否成功
     */
    @ApiOperation(value = "添加规格")
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> addSpecifications(@RequestBody SpecificationsAddRequest specificationsAddRequest, HttpServletRequest request){
        //判断是否为商家
        userService.isBusiness(request);

        if (specificationsAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Specifications specifications = new Specifications();
        BeanUtils.copyProperties(specificationsAddRequest,specifications);
        //校验请求体
        specificationsService.validSpecifications(specifications, false);
        //添加规格和属性
        List<AttributeAddRequest> attributeAddRequests = specificationsAddRequest.getAttributeAddRequests();
        Boolean aBoolean = specificationsService.addSpecifications(specifications, attributeAddRequests, request);
        ThrowUtils.throwIf(!aBoolean, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }

    /**·
     * 更新规格
     * @param specificationsUpdateRequest 更新请求体
     * @param request 网络请求
     * @return 是否成功
     */
    @ApiOperation(value = "更新规格")
    @PostMapping("/update")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> updateSpecifications(@RequestBody SpecificationsUpdateRequest specificationsUpdateRequest, HttpServletRequest request) {
        //判断是否为商家
        userService.isBusiness(request);

        if (specificationsUpdateRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Specifications specifications = new Specifications();
        BeanUtils.copyProperties(specificationsUpdateRequest,specifications);
        //校验
        specificationsService.validSpecifications(specifications,true);

        //更新规格和属性
        List<AttributeUpdateRequest> attributeUpdateRequests = specificationsUpdateRequest.getAttributeUpdateRequests();
        Boolean aBoolean = specificationsService.updateSpecifications(specifications, attributeUpdateRequests, request);
        ThrowUtils.throwIf(!aBoolean, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 根据菜品id获取规格脱敏信息
     * @param specificationsMyQueryRequest 请求体
     * @return 脱敏列表
     */
    @ApiOperation(value = "根据id获取规格")
    @PostMapping("/list/specificationsVOByComId")
    public BaseResponse<List<SpecificationsVO>> specificationsVOByComId(@RequestBody SpecificationsMyQueryRequest specificationsMyQueryRequest) {
        if (specificationsMyQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long commoditiesId = specificationsMyQueryRequest.getId();
        //根据id获取中间表数据
        QueryWrapper<SpecificationsCommodities> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("commoditiesId",commoditiesId);
        List<SpecificationsCommodities> list = specificationsCommoditiesService.list(queryWrapper);
        //根据中间表数据中的规格获取规格信息和属性
        return ResultUtils.success(specificationsCommoditiesService.getSpecificationsAddAttribute(list));
    }

    /**
     * 获取脱敏列表
     * @param specificationsQueryRequest 前端请求
     * @param request 网络请求
     * @return 脱敏列表
     */
    @ApiOperation(value = "获取规格列表")
    @PostMapping("/list/specificationsVO")
    public BaseResponse<List<SpecificationsVO>> specificationsVOList(@RequestBody SpecificationsQueryRequest specificationsQueryRequest, HttpServletRequest request) {
        //获取商家id
        Business loginBusiness = businessService.getLoginBusiness(request);
        Long businessId = loginBusiness.getId();
        //获取商家的规格表
        String specificationsName = specificationsQueryRequest.getSpecificationsName();
        QueryWrapper<Specifications> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(specificationsName), "specificationsName", specificationsName);
        queryWrapper.eq("businessId",businessId);
        List<Specifications> list = specificationsService.list(queryWrapper);
        //对规格信息脱敏
        List<SpecificationsVO> specificationsVO = specificationsService.getSpecificationsVO(list);
        //获取各规格的属性脱敏信息
        return ResultUtils.success(attributeService.getBySpecificationsVoId(specificationsVO));
    }
}
