package com.cj.jiaqingjiayi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cj.jiaqingjiayi.common.ErrorCode;

import com.cj.jiaqingjiayi.contant.CommonConstant;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.domain.Commodities;

import com.cj.jiaqingjiayi.model.domain.SpecificationsCommodities;
import com.cj.jiaqingjiayi.model.request.Commodities.CommoditiesQueryRequest;
import com.cj.jiaqingjiayi.model.vo.CommoditiesVO;
import com.cj.jiaqingjiayi.service.CommoditiesService;
import com.cj.jiaqingjiayi.mapper.CommoditiesMapper;
import com.cj.jiaqingjiayi.service.SpecificationsCommoditiesService;

import com.cj.jiaqingjiayi.utils.SqlUtils;
import org.apache.commons.lang3.StringUtils;

import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;


import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


/**
* @author 高木
* @description 针对表【commodities(商品表)】的数据库操作Service实现
* @createDate 2024-08-20 19:55:49
*/
@Service
public class CommoditiesServiceImpl extends ServiceImpl<CommoditiesMapper, Commodities>
    implements CommoditiesService{

    @Resource
    private SpecificationsCommoditiesService specificationsCommoditiesService;

    @Override
    public void validCommodities(Commodities commodities, Boolean update) {
        Long id = commodities.getId();
        Long commoditiesGroupId = commodities.getCommoditiesGroupId();
        String commoditiesName = commodities.getCommoditiesName();
        Double commoditiesPrice = commodities.getCommoditiesPrice();
        String status = commodities.getStatus();
        Integer inventoryStatus = commodities.getInventoryStatus();

        if (update) {
            if (id == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR);
            }
        }

        if (StringUtils.isAnyBlank(commoditiesName,status)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        if (commoditiesGroupId == null || commoditiesPrice == null || inventoryStatus == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
    }

    @Override
    public Long addCommodities(Commodities commodities) {
        //判断商品是否存在
        validCommoditiesName(commodities);
        //保存代码
        boolean save = this.save(commodities);
        ThrowUtils.throwIf(!save,ErrorCode.OPERATION_ERROR);
        return commodities.getId();
    }

    @Override
    public void removeByCommoditiesGroupId(Long commoditiesGroupId) {
        QueryWrapper<Commodities> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("commoditiesGroupId",commoditiesGroupId);
        List<Commodities> list = this.list(queryWrapper);
        //判断该分组是否有商品
        if (list.isEmpty()) {
            return;
        }
        boolean remove = this.remove(queryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);
        //删除中间表数据
        for (Commodities commodities : list) {
            Long commoditiesId = commodities.getId();
            specificationsCommoditiesService.deleteByCommoditiesId(commoditiesId);
        }
    }

    @Override
    public Boolean updateCommodities(Commodities commodities) {
        //判断商品是否存在
        validCommoditiesName(commodities);
        //更新商品
        return this.updateById(commodities);
    }

    @Override
    public CommoditiesVO getCommoditiesVO(Commodities commodities) {
        if (commodities == null) {
            return null;
        }
        long count = getCount(commodities);
        CommoditiesVO commoditiesVO = new CommoditiesVO();
        BeanUtils.copyProperties(commodities, commoditiesVO);
        if (count > 0) {
            commoditiesVO.setIsSpecification(1);
        }
        return commoditiesVO;
    }

    @Override
    public QueryWrapper<Commodities> getWrapper(CommoditiesQueryRequest commoditiesQueryRequest) {
        Long id = commoditiesQueryRequest.getId();
        Long businessId = commoditiesQueryRequest.getBusinessId();
        Long commoditiesGroupId = commoditiesQueryRequest.getCommoditiesGroupId();
        String status = commoditiesQueryRequest.getStatus();
        String commoditiesName = commoditiesQueryRequest.getCommoditiesName();
        String sortField = commoditiesQueryRequest.getSortField();
        String sortOrder = commoditiesQueryRequest.getSortOrder();
        QueryWrapper<Commodities> wrapper = new QueryWrapper<>();
        wrapper.eq(id != null, "id", id);
        wrapper.eq(businessId != null, "businessId", businessId);
        wrapper.eq(commoditiesGroupId != null, "commoditiesGroupId", commoditiesGroupId);
        wrapper.eq(StringUtils.isNotBlank(status), "status", status);
        wrapper.like(StringUtils.isNotBlank(commoditiesName), "commoditiesName", commoditiesName);
        wrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC), sortField);
        return wrapper;
    }

    @Override
    public List<CommoditiesVO> getCommoditiesVOList(List<Commodities> commoditiesVOList) {
        if (CollectionUtils.isEmpty(commoditiesVOList)) {
            return new ArrayList<>();
        }
        return commoditiesVOList.stream().map(item->{
            long count = getCount(item);
            CommoditiesVO commoditiesVO = new CommoditiesVO();
            BeanUtils.copyProperties(item,commoditiesVO);
            if (count > 0) {
                commoditiesVO.setIsSpecification(1);
            }
            return commoditiesVO;
        }).collect(Collectors.toList());
    }

    private void validCommoditiesName(Commodities commodities) {
        QueryWrapper<Commodities> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("commoditiesName",commodities.getCommoditiesName());
        queryWrapper.eq("businessId",commodities.getBusinessId());
        long count = this.count(queryWrapper);
        if (commodities.getId() == null) {
            ThrowUtils.throwIf(count > 0,ErrorCode.PARAMS_ERROR,"已存在该商品");
        }
        else {
            ThrowUtils.throwIf(count > 0,ErrorCode.PARAMS_ERROR,"已存在该商品");
        }
    }

    private long getCount(Commodities commodities) {
        Long commoditiesId = commodities.getId();
        QueryWrapper<SpecificationsCommodities> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("commoditiesId",commoditiesId);
        return specificationsCommoditiesService.count(queryWrapper);
    }

}




